// ***********************************************************************************************************************
// ** File:         arp.c                                                                                               **
// ** Title:        Routine for ARP Protocol                                                                            **
// ** CPU:          ATMega644                                                                                           **
// ** Created:      28.02.2008                                                                                          **
// ** Revised:      28.02.2008                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "arp.h"                                                            //
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
unsigned char arptablebuffer [10];                                          // Einfache Arp Tabelle fr einen eintrag
// ***********************************************************************************************************************

// Arp Pakete beantworten ************************************************************************************************
void arp()                                                                  //
{                                                                           //
    unsigned char i;                                                        //
    struct ETH_header *ETH_packet;                                          // ETH_packet zeigt auf den Struct ETH_header
    struct ARP_header *ARP_packet;                                          // ARP_packet zeigt auf den Struct ARP_header
    struct ARP_TABLE *ARP_table;                                            //
    ARP_table = (struct ARP_TABLE *)arptablebuffer;                         //
    ETH_packet = (struct ETH_header *)buffer;                               // Die ersten 14 Byte aus dem Buffer werden in ETH_packet gespeichert
    ARP_packet = (struct ARP_header *)&buffer[ETHERNET_HEADER_LENGTH];      // Die nchsten 28 Bytes aus dem Buffer werden in ARP_packet gespeichert
    switch ( ARP_packet->ARP_Opcode )                                       //
    {                                                                       //
        case 0x0100:                                                        // Request
            if ( ARP_packet->ARP_destIP != MyIP.LONG ) return ;             // wenn nicht fr meine IP dann abbrechen
            ARP_packet->ARP_Opcode = 0x0200;                                // Opcode auf Reply umstellen
            for ( i = 0; i < 10; i++ )                                      // mac und ip des senders in ziel kopieren
            {                                                               //
                ARP_packet->ARP_destMac[i] =                                // MAC und IP umkopieren
                ARP_packet->ARP_sourceMac[i];                               //
            }                                                               //
            for ( i = 0; i < 6 ; i++ )                                      // meine mac als absender einsetzen
            {                                                               //
                ARP_packet->ARP_sourceMac[i] =                              //
                MyMac [i];                                                  // MAC einsetzen
            }                                                               //
            ARP_packet->ARP_sourceIP = MyIP.LONG;                           // meine IP als absender einsetzen
            for( i = 0 ; i < 6 ; i++)                                       // sourceMAC in destMAC eintragen und meine MAC in sourceMAC kopieren
            {                                                               //
                ETH_packet->ETH_destMac[i] =                                // Die Absender MAC als neues ziel eintragen
                ETH_packet->ETH_sourceMac[i];                               //
                ETH_packet->ETH_sourceMac[i]=MyMac [i];                     // Neue Absender MAC ist unsere eigene MAC
            }                                                               //
            enc28j60PacketSend(ETHERNET_HEADER_LENGTH                       // ARP Reply senden
                + ARP_HEADER_LENGTH,buffer);                                //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("ARP->reply Send to: ");                           //
            print_uart_ip (ARP_packet->ARP_destIP);                         //
            #endif                                                          // DEBUG **************************************
        break;                                                              //
        case 0x0200:                                                        // Reply
            ARP_table->ARP_IP = ARP_packet->ARP_sourceIP;                   // Wenn eine ARP Antwort kommt dann die Absender IP in die Arp Tabelle eintragen
            for ( i = 0 ; i < 6 ; i++ )                                     //
            {                                                               //
                ARP_table->ARP_MAC[i] =                                     // Absender MAC in die ARP Tabelle eintragen
                ARP_packet->ARP_sourceMac[i];                               //
            }                                                               //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("ARP->Answer OK from: ");                          //
            print_uart_ip (ARP_packet->ARP_sourceIP);                       //
            #endif                                                          // DEBUG **************************************
        break;                                                              //
    }                                                                       //
}                                                                           //
// ***********************************************************************************************************************

// IP Adresse -> MAC Adresse *********************************************************************************************
unsigned char GetIP2MAC ( unsigned long IPadr )                             //
{                                                                           // Arp-request senden und schaun ob ip vorhanden
    unsigned int i;                                                         //
    struct ETH_header *ETH_packet;                                          //
    struct ARP_header *ARP_packet;                                          //
    struct ARP_TABLE *ARP_table;                                            //
    ARP_table = (struct ARP_TABLE *)arptablebuffer;                                             // Ist die ARP Tabelle fr einen Request
    ETH_packet = (struct ETH_header *)buffer;                                                    // Die ersten 14 Byte aus dem Buffer werden in ETH_packet gespeichert
    ARP_packet = (struct ARP_header *)&buffer[ETHERNET_HEADER_LENGTH];                           // Die nchsten 28 Bytes aus dem Buffer werden in ARP_packet gespeichert
                                                                            //
    ARP_table->ARP_IP = 0x00000000;                                         // Arp Table lschen
    ARP_packet->HWtype = 0x0100;                                            // Hardware Type Ethernet    
    ARP_packet->Protocoltype = 0x0008;                                      // Protocoltype = IP
    ARP_packet->HWsize = 0x06;                                              // Hardware adresse size 6
    ARP_packet->Protocolsize = 0x04;                                        // Protocolsize 4
    ARP_packet->ARP_Opcode = 0x0100;                                        // Opcode fr request
    for ( i = 0; i < 6; i++ )                                               // Destination MAC und Source MAC einsetzen
    {                                                                       //
        ARP_packet->ARP_destMac[i] = 0x00;                                  // Destination MAC auf 0 setzen
        ARP_packet->ARP_sourceMac[i] = MyMac [i];                           // Source MAC einsetzen
        ETH_packet->ETH_destMac[i] = 0xff;                                  // Destination MAC in Ethernetframe eintragen
        ETH_packet->ETH_sourceMac[i] = MyMac [i];                           // Source MAC in Ethernetframe eintragen
    }                                                                       //
    ARP_packet->ARP_destIP = IPadr;                                         // Destination IP einsetzen
    ARP_packet->ARP_sourceIP = MyIP.LONG;                                   // Source IP einsetzen
    ETH_packet->ETH_typefield = 0x0608;                                     //
    enc28j60PacketSend( ETHERNET_HEADER_LENGTH + ARP_HEADER_LENGTH,buffer); // Buffer senden
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_debug ("GETIP2MAC->Request Send to ");                            //
    print_uart_ip (ARP_packet->ARP_destIP);                                 //
    #endif                                                                  // DEBUG **************************************
    deltasec = 0;                                                           // Timeout Timer starten
    while ( 1 )                                                             //
    {                                                                       //
        ethernet();                                                         // Prfe ob ein Paket angekommen ist
        if ((ARP_table->ARP_IP == IPadr) && ( deltasec < TimeOut))          // Wenn die angefragte IP = angefragte IP und Timeout nicht erreicht ist
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("GETIP2MAC->Request OK from: ");                   //
            print_uart_ip (ARP_table->ARP_IP);                              //
            #endif                                                          // DEBUG **************************************
            return( ARP_ANSWER );                                           // Antwort auf unser Arp Paket angekommen
        }                                                                   //
        if (deltasec >= TimeOut)                                            // Wenn der Timeout timer abgelaufen ist 
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("GETIP2MAC->Request Timeout for: ");               //
            print_uart_ip (IPadr);                                          //
            #endif                                                          // DEBUG **************************************
            return( NO_ARP_ANSWER );                                        // Keine antwort bekommen
        }                                                                   //
    }                                                                       //
}                                                                           //
// ***********************************************************************************************************************

