// ***********************************************************************************************************************
// ** File:         crc.c                                                                                               **
// ** Title:        Routine for Checksum                                                                                **
// ** CPU:          ATMega644                                                                                           **
// ** Created:      28.02.2008                                                                                          **
// ** Revised:      28.02.2008                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "crc.h"                                                            //
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// 32 Bit wort umdrehen **************************************************************************************************
unsigned long ChangeEndian32bit( unsigned long Wert)                        // Einige Funktionen die AVR-GCC nicht hat wie das tauschen
{                                                                           // der Bytes in einen int oder long. Big-Endian <-> Little-Endian.
    Wert = ((Wert & 0xff000000)>>24) | ((Wert & 0x000000ff)<<24)            // Tauschen von High und Low Byte 
    | ((Wert & 0x00ff0000)>>8) | ((Wert & 0x0000ff00)<<8);                  // Tauschen von zweiten High und zweiten Low Byte
    return( Wert );                                                         //
}                                                                           //
// ***********************************************************************************************************************

// 16 Bit wort umdrehen **************************************************************************************************
unsigned int ChangeEndian16bit( unsigned int Wert)                          // Tauschen von High und Low Byte
{                                                                           //
    Wert = ( ( Wert & 0xff00 ) >> 8 ) | ( ( Wert & 0x00ff ) << 8 );         //
    return( Wert );                                                         //
}                                                                           //
// ***********************************************************************************************************************

// 16 Bit CRC berechnen **************************************************************************************************
int Checksum_16 (unsigned char * pointer, unsigned int headerlenght)        // Berechnung der 16.Bit Checksumme eines Speicherbereiches im RAM.
{                                                                           // Benotig z.B. fuer die Checksumme von Datenpacketen wie in TCP/IP.
    unsigned long checksum = 0x00;                                          //
    unsigned int result;                                                    //
    unsigned char DataH;                                                    //
    unsigned char DataL;                                                    //
    while( headerlenght > 1)                                                // Jetzt werden alle Packete in einer While Schleife addiert
    {                                                                       //
        DataH=*pointer++;                                                   //
        DataL=*pointer++;                                                   //
        result =~ ((DataH << 8)+ DataL);                                    //
        checksum = checksum + result;                                       //
        headerlenght -=2 ;                                                  // decrimiert Lnge von TCP Headerschleife um 2
    }                                                                       //
    if( headerlenght > 0)                                                   // Ist der Wert result16 ungerade ist DataL = 0
    {                                                                       //
        DataH=*pointer;                                                     //
        result =~ (DataH << 8);                                             //
        checksum = checksum + result;                                       //
    }                                                                       //
    checksum = ((checksum&0x0000FFFF)+((checksum&0xFFFF0000)>>16));         // Komplementbildung (addiert Long INT_H Byte mit Long INT L Byte)
    checksum = ((checksum&0x0000FFFF)+((checksum&0xFFFF0000)>>16));         //
    checksum = (checksum & 0x0000FFFF);                                     //
    return (checksum);                                                      //
}                                                                           //
// ***********************************************************************************************************************
