// ***********************************************************************************************************************
// ** File:         dhcp.c                                                                                              **
// ** Title:        Routine for DHCP Protocol                                                                           **
// ** CPU:          AT89C51ED2                                                                                          **
// ** Created:      23.04.2007                                                                                          **
// ** Revised:      23.04.2007                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "dhcp.h"                                                           //
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
unsigned char dhcpconf [24];                                                // Temporrer Buffer fr DHCP
unsigned char DHCP_Message_Type = 0;                                        // DHCP Antwort wird hier gespeichert
// ***********************************************************************************************************************

// DHCP Header bauen *****************************************************************************************************
void DHCP_buildheader()                                                     //
{                                                                           //
    unsigned int i;                                                         //
    struct DHCP_HEADER * dhcpc;                                             // TCP struct anlegen
    dhcpc = (struct DHCP_HEADER *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH + UDP_HEADER_LENGHT];//
    dhcpc->Messages_Type = BOOT_REQUEST;                                    //
    dhcpc->Hardware_Type = HARDWARE_TYPE_ETHERNET;                          //
    dhcpc->Hardware_Adress_Lenght = 6;                                      //
    dhcpc->Hops = 0;                                                        //
    dhcpc->Transaction_ID = 0x12345678;                                     //
    dhcpc->Second_elapsed = 0;                                              //
    dhcpc->Bootp_Flags = 0;                                                 //
    dhcpc->Client_IP.LONG = 0;                                              //
    dhcpc->Your_IP.LONG = 0;                                                //
    dhcpc->Server_IP.LONG = 0;                                              //
    dhcpc->Relay_IP.LONG = 0;                                               //
    for ( i = 0 ; i < 6 ; i++ )                                             //
    {                                                                       //
        dhcpc->Your_MACADRESS[i] = MyMac[i];                                //
    }                                                                       //
    for ( i = 0 ; i < SERVERNAME_LENGHT ; i++ )                             //
    {                                                                       //
        dhcpc->Servername[i] = 0 ;                                          //
    }                                                                       //
    for ( i = 0 ; i < BOOT_FILENAME_LENGHT ; i++ )                          //
    {                                                                       //
        dhcpc->Boot_Filename[i] = 0;                                        //
    }                                                                       //
    dhcpc->Magic_Cookie = MAGIC_COOKIE;                                     //
    for ( i = 0 ; i < OPTION_LENGHT ; i++ )                                 //
    {                                                                       //
        dhcpc->Options[i] = 0;                                              //
    }                                                                       //
    dhcpc->Options[0] = 0xff;                                               //
}                                                                           //
// ***********************************************************************************************************************

// DHCP Optionen hinzufgen **********************************************************************************************
unsigned int DHCP_AddOption( unsigned char Option, unsigned long ExtraData, unsigned char * Optionfield )
{                                                                           //
    unsigned int i = 0 ;                                                    //
    while( i < OPTION_LENGHT )                                              //
    {                                                                       //
        if ( Optionfield[i] == 0xff )                                       // suche Ende (0xff) vom Optionfeld mit letzten eintrag
        {                                                                   //
            break;                                                          //
        }                                                                   //
        else                                                                //
        {                                                                   //
            i++;                                                            // um 1 erhhen um die lnge des Optioneintrages zu ermitteln
            i = i + Optionfield[i] + 1;                                     // i um ( lnge + 1 ) des Optioneintrages erhhen, sollte dann auf den nchsten Eintrag zeigen
        }                                                                   //
    }                                                                       //
    switch( Option )                                                        //
    {                                                                       //
        case Option_DHCP_MESSAGES_TYPE:                                     //
            Optionfield[ i++ ] = Option_DHCP_MESSAGES_TYPE;                 //
            Optionfield[ i++ ] = 1;                                         // Lnge dieses Optionseintrages
            Optionfield[ i++ ] = ExtraData;                                 //
        break;                                                              //
        case Option_SERVER_ID:                                              //
            Optionfield[ i++ ] = Option_SERVER_ID;                          //
            Optionfield[ i++ ] = 4;                                         // Lnge dieses Optionseintrages
            Optionfield[ i++ ] = ExtraData;                                 //
            Optionfield[ i++ ] = ExtraData>>8;                              //
            Optionfield[ i++ ] = ExtraData>>16;                             //
            Optionfield[ i++ ] = ExtraData>>24;                             //
        break;                                                              //
        case Option_REQUESTET_IP_ADRESS:                                    //
            Optionfield[ i++ ] = Option_REQUESTET_IP_ADRESS;                //
            Optionfield[ i++ ] = 4;                                         // Lnge dieses Optionseintrages
            Optionfield[ i++ ] = ExtraData;                                 //
            Optionfield[ i++ ] = ExtraData>>8;                              //
            Optionfield[ i++ ] = ExtraData>>16;                             //
            Optionfield[ i++ ] = ExtraData>>24;                             //
        break;                                                              //
        case Option_OPTIONLIST:                                             //
            Optionfield[ i++ ] = Option_OPTIONLIST;                         //
            Optionfield[ i++ ] = 4;                                         // Lnge dieses Optionseintrages
            Optionfield[ i++ ] = Option_SUBNETMASK;                         //
            Optionfield[ i++ ] = Option_DOMAIN_NAME;                        //
            Optionfield[ i++ ] = Option_ROUTER;                             //
            Optionfield[ i++ ] = Option_DOMAIN_NAME_SRVER;                  //
        break;                                                              //
        case Option_CLIENT_IDENTIFIER:                                      //
            Optionfield[ i++ ] = Option_CLIENT_IDENTIFIER;                  //
            Optionfield[ i++ ] = 7;                                         // Lnge dieses Optionseintrages
            Optionfield[ i++ ] = 1;                                         // Type Ethernet
            Optionfield[ i++ ] = MyMac [0];                                 // Meine MAC Adr.
            Optionfield[ i++ ] = MyMac [1];                                 // Meine MAC Adr.
            Optionfield[ i++ ] = MyMac [2];                                 // Meine MAC Adr.
            Optionfield[ i++ ] = MyMac [3];                                 // Meine MAC Adr.
            Optionfield[ i++ ] = MyMac [4];                                 // Meine MAC Adr.
            Optionfield[ i++ ] = MyMac [5];                                 // Meine MAC Adr.
        break;                                                              //
    }                                                                       //
    Optionfield[ i++ ] = 0xff;                                              // Letzer eintrag im Optionsfeld muss FF sein
    return( i );                                                            // Gibt das ende das Optionsfeldes zurck
}                                                                           //
// ***********************************************************************************************************************

// DHCP Optionen aus empfangenen Paket auslesen **************************************************************************
void DHCP_PharseOption( unsigned char * Optionfield )                       //
{                                                                           //
    unsigned int OptionPointer = 0;                                         //
    struct DHCP_CONFIG * DHCPConfig;                                        //
    DHCPConfig = (struct DHCP_CONFIG *)dhcpconf;                                                  //
    while( OptionPointer < OPTION_LENGHT )                                  //
    {                                                                       //
        if ( Optionfield[ OptionPointer ] == 0xff )                         // suche Ende (0xff) vom Optionfeld mit letzten eintrag eintrag
        {                                                                   //
            break;                                                          //
        }                                                                   //
        else                                                                //
        {                                                                   //
            switch ( Optionfield[ OptionPointer++ ] )                       //
            {                                                               //
                case Option_SUBNETMASK:                                     //
                    memcpy( &DHCPConfig->Subnetmask.LONG ,                  //
                        &Optionfield[ OptionPointer + 1 ], 4);              //
                    OptionPointer = OptionPointer +                         //
                        Optionfield[ OptionPointer++ ];                     //
                break;                                                      //
                case Option_ROUTER:                                         //
                    memcpy( &DHCPConfig->Router_IP.LONG ,                   //
                        &Optionfield[ OptionPointer + 1 ], 4);              //
                    OptionPointer = OptionPointer +                         //
                        Optionfield[ OptionPointer++ ];                     //
                break;                                                      //
                case Option_DOMAIN_NAME_SRVER:                              //
                    memcpy( &DHCPConfig->DNS_IP.LONG ,                      //
                        &Optionfield[ OptionPointer + 1 ], 4);              //
                    OptionPointer = OptionPointer +                         //
                        Optionfield[ OptionPointer++ ];                     //
                break;                                                      //
                case Option_IP_ADRESS_LEASETIME:                            //
                    memcpy( &DHCPConfig->Leasetime.LONG ,                   //
                        &Optionfield[ OptionPointer + 1 ], 4);              //
                    OptionPointer = OptionPointer +                         //
                        Optionfield[ OptionPointer++ ];                     //
                break;                                                      //
                case Option_SERVER_ID:                                      //
                    memcpy( &DHCPConfig->Server_IP.LONG ,                   //
                        &Optionfield[ OptionPointer + 1 ], 4);              //
                    OptionPointer = OptionPointer +                         //
                        Optionfield[ OptionPointer++ ];                     //
                break;                                                      //
                case Option_DHCP_MESSAGES_TYPE:                             //
                    DHCP_Message_Type =                                     //
                        Optionfield[ OptionPointer + 1 ];                   //
                    OptionPointer = OptionPointer +                         //
                        Optionfield[ OptionPointer++ ];                     //
                break;                                                      //
                default:                                                    //
                    OptionPointer = OptionPointer +                         //
                        Optionfield[ OptionPointer++ ];                     //
            }                                                               //
        }                                                                   //
    }                                                                       //
}                                                                           //
// ***********************************************************************************************************************

// ***********************************************************************************************************************
void DHCPAnswer()                                                           //
{                                                                           //
    struct DHCP_HEADER * dhcpc;                                             // TCP struct anlegen
    dhcpc = (struct DHCP_HEADER *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH + UDP_HEADER_LENGHT];//
    DHCP_PharseOption( dhcpc->Options );                                    //
}                                                                           //
// ***********************************************************************************************************************

// ***********************************************************************************************************************
unsigned int DHCP_SendDiscover()                                            //
{                                                                           //
    struct DHCP_HEADER * dhcpc;                                             // TCP struct anlegen
    struct DHCP_CONFIG * DHCPConfig;                                        //
    dhcpc = (struct DHCP_HEADER *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH + UDP_HEADER_LENGHT];//
    DHCPConfig = (struct DHCP_CONFIG *)dhcpconf;                                                  //
    DHCP_buildheader();                                                     // DHCP Header bauen
    DHCP_AddOption( Option_DHCP_MESSAGES_TYPE, DHCP_DISCOVER, dhcpc->Options );// Optionsfeld auf DHCP Discover setzen
    DHCP_AddOption( Option_CLIENT_IDENTIFIER,0,dhcpc->Options );            //
    DHCP_AddOption( Option_OPTIONLIST, 0, dhcpc->Options );                 // Optionsfeld auf
    udp_send( DestMACAddress, 0xffffffff, 0x4400, 0x4300, DHCP_HEADER_LENGHT);// DHCP Discover senden
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_debug ("DHCP_SENDDISCOVER->Send\r\n");                            //
    #endif                                                                  // DEBUG **************************************
    deltasec = 0;                                                           // Timeout Timer starten
    while ( 1 )                                                             //
    {                                                                       //
        ethernet();                                                         // Prfe ob ein Paket angekommen ist
        if ((DHCP_Message_Type == DHCP_OFFER ) && ( deltasec < TimeOut))    // Wenn die angefragte IP = angefragte IP und Timeout nicht erreicht ist
        {                                                                   //
            DHCPConfig->Client_IP.LONG = dhcpc->Your_IP.LONG;               //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("DHCP_SENDDISCOVER->OK\r\n");                      //
            #endif                                                          // DEBUG **************************************
            return( DHCPC_OK );                                             // Antwort auf unser Arp Paket angekommen
        }                                                                   //
        if (deltasec >= TimeOut)                                            // Wenn der Timeout timer abgelaufen ist
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("DHCP_SENDDISCOVER->Timeout\r\n");                 //
            #endif                                                          // DEBUG **************************************
            return( DHCPC_TIMEOUT );                                        // Keine antwort bekommen
        }                                                                   //
    }                                                                       //
}                                                                           //
// ***********************************************************************************************************************

// ***********************************************************************************************************************
unsigned int DHCP_SendRequest()                                             //
{                                                                           //
    struct DHCP_HEADER * dhcpc;                                             // TCP struct anlegen
    struct DHCP_CONFIG * DHCPConfig;                                        //
    dhcpc = (struct DHCP_HEADER *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH + UDP_HEADER_LENGHT];//
    DHCPConfig = (struct DHCP_CONFIG *)dhcpconf;                                                  //
    DHCP_buildheader();                                                     //
    DHCP_AddOption( Option_DHCP_MESSAGES_TYPE, DHCP_REQUEST, dhcpc->Options );//
    DHCP_AddOption( Option_CLIENT_IDENTIFIER,0,dhcpc->Options );            //
    DHCP_AddOption( Option_REQUESTET_IP_ADRESS, DHCPConfig->Client_IP.LONG ,//
        dhcpc->Options );                                                   //
    DHCP_AddOption( Option_SERVER_ID, DHCPConfig->Server_IP.LONG , dhcpc->Options );//
    DHCP_AddOption( Option_OPTIONLIST, 0, dhcpc->Options );                 //
    udp_send( DestMACAddress, 0xffffffff, 0x4400, 0x4300, DHCP_HEADER_LENGHT);// DHCP Req senden
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_debug ("DHCP_SENDREQUEST->Send\r\n");                             //
    #endif                                                                  // DEBUG **************************************
    deltasec = 0;                                                           // Timeout Timer starten
    while ( 1 )                                                             //
    {                                                                       //
        ethernet();                                                         // Prfe ob ein Paket angekommen ist
        if ((DHCP_Message_Type == DHCP_ACK ) && ( deltasec < TimeOut))      // Wenn die angefragte IP = angefragte IP und Timeout nicht erreicht ist
        {                                                                   //
            DHCPConfig->Client_IP.LONG = dhcpc->Your_IP.LONG;               //
            MyIP.LONG = DHCPConfig->Client_IP.LONG;                         // Meine neue IP
            Netmask.LONG = DHCPConfig->Subnetmask.LONG;                     // Meine neue Netmask
            Gateway.LONG = DHCPConfig->Router_IP.LONG;                      // IP des Gateway
            DNSserver.LONG = DHCPConfig->DNS_IP.LONG;                       // DNS Server IP
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("DHCP_SENDREQUEST->New IP :");                     //
            print_uart_ip (MyIP.LONG);                                      //
            print_debug ("DHCP_SENDREQUEST->New Subnet Mask :");            //
            print_uart_ip (Netmask.LONG);                                   //
            print_debug ("DHCP_SENDREQUEST->New Gateway :");                //
            print_uart_ip (Gateway.LONG);                                   //
            print_debug ("DHCP_SENDREQUEST->New DNS Server :");             //
            print_uart_ip (DNSserver.LONG);                                 //
            print_debug ("DHCP_SENDREQUEST->OK\r\n");                       //
            #endif                                                          // DEBUG **************************************
            return( DHCPC_OK );                                             // Antwort auf unser Arp Paket angekommen
        }                                                                   //
        if (deltasec >= TimeOut)                                            // Wenn der Timeout timer abgelaufen ist
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("DHCP_SENDREQUEST->Timeout\r\n");                  //
            #endif                                                          // DEBUG **************************************
            return( DHCPC_TIMEOUT );                                        // Keine antwort bekommen
        }                                                                   //
    }                                                                       //
}                                                                           //
// ***********************************************************************************************************************

// ***********************************************************************************************************************
unsigned int DHCP_GetConfig( void )                                         //
{                                                                           //
    unsigned long SaveIP;                                                   //
    SaveIP = MyIP.LONG;                                                     //
    MyIP.LONG = 0x00000000;                                                 //
    DestMACAddress[0]=0xff;                                                 //
    DestMACAddress[1]=0xff;                                                 //
    DestMACAddress[2]=0xff;                                                 //
    DestMACAddress[3]=0xff;                                                 //
    DestMACAddress[4]=0xff;                                                 //
    DestMACAddress[5]=0xff;                                                 //
    DHCP_Message_Type = 0;                                                  //
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_debug("DHCP_GETCONFIG->DHCP SendDiscover\r\n");                   //
    #endif                                                                  // DEBUG **************************************
    if ( DHCP_SendDiscover() == DHCPC_TIMEOUT )                             // DHCPC-Discover erzeugen und senden
    {                                                                       //
        MyIP.LONG = SaveIP;                                                 //
        return( DHCPC_TIMEOUT );                                            //
    }                                                                       //
    DHCP_Message_Type = 0;                                                  //
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_debug("DHCP_GETCONFIG->DHCP SendRequest\r\n");                    //
    #endif                                                                  // DEBUG **************************************
    if ( DHCP_SendRequest() == DHCPC_TIMEOUT )                              // DHCPC-Request erzeugen ud senden
    {                                                                       //
        MyIP.LONG = SaveIP;                                                 //
        return( DHCPC_TIMEOUT );                                            //
    }                                                                       //
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_debug("DHCP_GETCONFIG->DHCP OK\r\n");                             //
    #endif                                                                  // DEBUG **************************************
    return ( DHCPC_OK );                                                    //
}                                                                           //
// ***********************************************************************************************************************
