// ***********************************************************************************************************************
// ** File:         dhcp.h                                                                                              **
// ** Title:        Routine for DHCP Protocol                                                                           **
// ** CPU:          AT89C51ED2                                                                                          **
// ** Created:      23.04.2007                                                                                          **
// ** Revised:      23.04.2007                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// COMPILER INFORMATION **************************************************************************************************
#ifndef DHCP_H                                                              //
#define DHCP_H                                                              //
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "enc28j60.h"                                                       //
#include "ethernet.h"                                                       //
#include "ip.h"                                                             //
#include "udp.h"                                                            //
#include "global.h"                                                         //
#include <string.h>
#include <stdlib.h>
//#include "debug.h"                                                          //
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
#define DHCP_HEADER_LENGHT          336                                     //
#define DHCP_CONFIG_LENGHT          24                                      //
#define DHCP_REQUEST_TIMEOUT        1000                                    // in 1/100 sekunden
#define SERVERNAME_LENGHT           64                                      //
#define BOOT_FILENAME_LENGHT        128                                     //
#define OPTION_LENGHT               96                                      //
#define MAGIC_COOKIE                0x63538263                              // Das Magic-Cookie, markiert den Anfang des Optionblocks
#define Option_SUBNETMASK           1                                       //
#define Option_TIME_OFFSET          2                                       //
#define Option_ROUTER               3                                       //
#define Option_DOMAIN_NAME_SRVER    6                                       //
#define Option_HOST_NAME            12                                      //
#define Option_DOMAIN_NAME          15                                      //
#define Option_BROADCAST_ADRESS     28                                      //
#define Option_REQUESTET_IP_ADRESS  50                                      //
#define Option_IP_ADRESS_LEASETIME  51                                      //
#define Option_DHCP_MESSAGES_TYPE   53                                      //
#define Option_SERVER_ID            54                                      //
#define Option_OPTIONLIST           55                                      //
#define Option_VENDOR_CLASS         60                                      //
#define Option_CLIENT_IDENTIFIER    61                                      //
#define Option_CLIENT_FULL_DOMAIN   81                                      //
#define Option_AUTOCONFIG           116                                     //
#define Option_END                  255                                     //
#define DHCP_DISCOVER               1                                       //
#define DHCP_OFFER                  2                                       //
#define DHCP_REQUEST                3                                       //
#define DHCP_ACK                    5                                       //
#define BOOT_REQUEST                1                                       //
#define BOOT_REPLY                  2                                       //
#define DHCPC_OK                    0                                       //
#define DHCPC_TIMEOUT               0xffff                                  //
#define HARDWARE_TYPE_ETHERNET      1                                       //
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
extern unsigned char dhcpconf [24];                                         // Temporrer Buffer fr DHCP
extern unsigned char DHCP_Message_Type;                                     // DHCP Antwort wird hier gespeichert
// ***********************************************************************************************************************

// Temporrer speicher fr die DHCP Antwort ******************************************************************************
struct DHCP_CONFIG                                                          //
{                                                                           //
    union LONG_BYTE Client_IP;                                              // Speichert die eigene IP die Zugewiesen wird.
    union LONG_BYTE Router_IP;                                              // Speichert die IP des naechsten Gateway.
    union LONG_BYTE Subnetmask;                                             // Speichert die zugewiesene Submetmask.
    union LONG_BYTE DNS_IP;                                                 // Speichert die IP des zugewiesenen DNS-Server.
    union LONG_BYTE Server_IP;                                              // Speichert die Server-ID.
    union LONG_BYTE Leasetime;                                              // Speichert die Leasetime.
};                                                                          //
// ***********************************************************************************************************************

// Struct fr der DHCP Header ********************************************************************************************
struct DHCP_HEADER                                                          //
{                                                                           //
    unsigned char Messages_Type;                                            // Zeigt an ob req. oder response
    unsigned char Hardware_Type;                                            // 0x01 = Ethernet
    unsigned char Hardware_Adress_Lenght;                                   // 0x06 = normale MAC Adresse
    unsigned char Hops;                                                     //
    unsigned long Transaction_ID;                                           // Willkrliche Transaktionsnummer
    unsigned int Second_elapsed;                                            // Uptime des DHCP Clients
    unsigned int Bootp_Flags;                                               //
    union LONG_BYTE Client_IP;                                              // Client IP Adresse
    union LONG_BYTE Your_IP;                                                // Eigene IP Adresse
    union LONG_BYTE Server_IP;                                              // Server IP Adresse
    union LONG_BYTE Relay_IP;                                               // Relay Agent IP Adresse
    unsigned char Your_MACADRESS[16];                                       // Client MAC Adresse
    unsigned char Servername[64];                                           // Servername des DHCP Servers
    unsigned char Boot_Filename[128];                                       // Boot Datei die angefordert wird
    unsigned long Magic_Cookie;                                             // Kennzeichnet den start des Optionsfeldes
    unsigned char Options[96];                                              // Optionsfeld
};                                                                          //
// ***********************************************************************************************************************

// FUNCTIONS *************************************************************************************************************
void DHCP_buildheader(void);                                                //
unsigned int DHCP_AddOption( unsigned char Option, unsigned long ExtraData, unsigned char * Optionfield );
void DHCP_PharseOption( unsigned char * Optionfield );                      //
void DHCPAnswer(void);                                                      //
unsigned int DHCP_SendDiscover(void);                                       //
unsigned int DHCP_SendRequest(void);                                        //
unsigned int DHCP_GetConfig( void );                                        //
// ***********************************************************************************************************************

#endif
