// ***********************************************************************************************************************
// ** File:         dns.c                                                                                               **
// ** Title:        Routine for DNS Protocol                                                                            **
// ** CPU:          AT89C51ED2                                                                                          **
// ** Created:      23.04.2007                                                                                          **
// ** Revised:      23.04.2007                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "dns.h"                                                            //
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
union LONG_BYTE Netmask;                                                    // Die eigene SubnetMask
union LONG_BYTE Gateway;                                                    // Die Gateway Adresse
union LONG_BYTE DNSserver;                                                  // Die DNS Server Adresse
unsigned long DNS_Answ_IP = 0;                                              // Nach einem DNS Request steht hier die IP der angefragten adresse
// ***********************************************************************************************************************

// DNS Request senden ****************************************************************************************************
unsigned char DNSRequest(char *HOSTNAME)                           //
{                                                                           //
    unsigned int i;                                                         //
    struct DNS_request * DNS_packet;                                        // DNS_struct anlegen
    struct ARP_TABLE *ARP_table;                                            // ARP Tabelle Struct anlegen
    DNS_packet = (struct DNS_request *)&buffer[ETHERNET_HEADER_LENGTH+IP_HEADER_LENGTH+UDP_HEADER_LENGHT];// DNS_packet start @ buffer[42]
    ARP_table = (struct ARP_TABLE *)arptablebuffer;                                             // Buffer fr IP2MAC Funktion
    GetIP2MAC(Gateway.LONG);                                                // MAC von Gateway ermitteln
    DNS_packet->TransactionID = 0x3412;                                     // Irgendeine Nummer die die Transaktion Numeriert
    DNS_packet->Flags = 0x0001;                                             // 0x0100 = request
    DNS_packet->Questions = 0x0100;                                         // 0x0001 = eine anfrage
    DNS_packet->Answer_RRs = 0x0000;                                        // 0x0000 =
    DNS_packet->Authority_RRs = 0x0000;                                     // 0x0000 =
    DNS_packet->Additional_RRs = 0x0000;                                    // 0x0000 =         // 12 Byte
    i = DNS_convertHostName(HOSTNAME);                                      // Der Hostname wird in eine DNS Namestring gewandelt
    buffer[54+i+1] = '\0';                                                  // Nach dem DNS Namen gehrt noch eine
    buffer[54+i+2] = 1;                                                     // kombination aus 0101
    buffer[54+i+3] = '\0';                                                  //
    buffer[54+i+4] = 1;                                                     //
    udp_send( ARP_table->ARP_MAC, DNSserver.LONG, 0x0800, 0x3500, 12 + i + 5);// DNS Anfrage auf port 53 senden
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_uart_clock();                                                     // Uhr ber Uart ausgeben
    printf_fast ("DNSREQUEST->DNS Request %s send to IP-Addr :",HOSTNAME);  // Debug ausgabe
    print_uart_ip (DNSserver.LONG);                                         //
    #endif                                                                  // DEBUG **************************************
    deltasec = 0;                                                           // Timeout Timer starten
    DNS_Answ_IP = 0xffffffff;                                               //
    while ( 1 )                                                             //
    {                                                                       //
        ethernet();                                                         // Prfe ob ein Paket angekommen ist
        if ((DNS_Answ_IP != 0xffffffff) && ( deltasec < TimeOut))           // Wenn die angefragte IP = angefragte IP und Timeout nicht erreicht ist
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_uart_clock();                                             // Uhr ber Uart ausgeben
            printf_fast ("DNSREQUEST->DNS Request %s OK ",HOSTNAME);        // Debug ausgabe
            printf_fast ("IP-Addr :");                                      //
            print_uart_ip (DNS_Answ_IP);                                    //
            #endif                                                          // DEBUG **************************************
            return( DNS_ANSWER );                                           // Antwort auf unser Arp Paket angekommen
        }                                                                   //
        if (deltasec >= TimeOut)                                            // Wenn der Timeout timer abgelaufen ist
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_uart_clock();                                             // Uhr ber Uart ausgeben
            printf_fast ("DNSREQUEST->DNS Request %s Timeout\r\n",HOSTNAME);// Debug ausgabe
            #endif                                                          // DEBUG **************************************
            return( DNS_NO_ANSWER );                                        // Keine antwort bekommen
        }                                                                   //
    }                                                                       //
}                                                                           //
// ***********************************************************************************************************************

// DNS Answer auswerten **************************************************************************************************
void DNSAnswer(void)                                                        //
{                                                                           //
    char *Answ_String;                                                      //
    struct DNS_request * DNS_packet;                                        // DNS_struct anlegen
    struct DNS_answer * DNS_Answ_packet;                                    //
    DNS_packet = (struct DNS_request *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH +        // DNS_packet start @ buffer[42]
        UDP_HEADER_LENGHT];                                                 //
    Answ_String = &buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH +       // DNS Answer Name start @ buffer [54]
        UDP_HEADER_LENGHT + 12];                                            //
    DNS_Answ_packet = (struct DNS_answer *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH +   // DNS Answer Start @ buffer [59 + len Answer Name]
        UDP_HEADER_LENGHT + 12 + strlen(Answ_String)+5];                    //
    if (DNS_Answ_packet->Type != A_HOSTNAME)                                // wenn im ersten teil der antwort keine ip adresse
    {                                                                       // ist dann im zweiten teil der antwort nachsehen
        DNS_Answ_packet = (struct DNS_answer *)&buffer[54 + strlen(Answ_String) + 5 +            //
            ChangeEndian16bit( DNS_Answ_packet->Datalenght ) +              //
            DNS_ANSWER_HEADER_LENGHT ];                                     //
    }                                                                       //
    DNS_Answ_IP = DNS_Answ_packet->Adress;                                  //
    return;                                                                 //
}                                                                           //
// ***********************************************************************************************************************

// DNS Namen Konvertieren ************************************************************************************************
unsigned int DNS_convertHostName( char * HOSTNAME)                 //
{                                                                           //
    int i,j=0,k;                                                            //
    int strlaenge = strlen( HOSTNAME );                                     //
    k = strlaenge;                                                          //
                                                                            //
    for ( i = strlaenge - 1 ; i >= 0 ; i-- )                                //
    {                                                                       //
        if ( HOSTNAME[i] == '.' )                                           //
        {                                                                   //
            buffer[54 + k] = j;                                             //
            k--;                                                            //
            j = 0;                                                          //
        }                                                                   //
        else                                                                //
        {                                                                   //
            buffer[54 + k] = HOSTNAME[i];                                   //
            k--;                                                            //
            j++;                                                            //
        }                                                                   //
    }                                                                       //
    buffer[54 + k] = j;                                                     //
    buffer[ 54 + strlaenge + 1 ] = '\0';                                    //
    return( strlaenge + 1 );                                                //
}                                                                           //
// ***********************************************************************************************************************
