// ***********************************************************************************************************************
// ** File:         dns.h                                                                                               **
// ** Title:        Routine for DNS Protocol                                                                            **
// ** CPU:          AT89C51ED2                                                                                          **
// ** Created:      23.04.2007                                                                                          **
// ** Revised:      23.04.2007                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// COMPILER INFORMATION **************************************************************************************************
#ifndef DNS_H                                                               //
#define DNS_H                                                               //
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "arp.h"                                                            //
#include "ethernet.h"                                                       //
#include "udp.h"                                                            //
#include "global.h"                                                         //
//#include "debug.h"                                                          //
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
#define DNS_HEADER_LENGHT               12                                  //
#define DNS_ANSWER_HEADER_LENGHT        12                                  //
#define A_HOSTNAME                      0x0100                              //
#define DNS_ANSWER                      0                                   //
#define DNS_NO_ANSWER                   !DNS_ANSWER                         //
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
extern union LONG_BYTE Netmask;                                             // Die eigene SubnetMask
extern union LONG_BYTE Gateway;                                             // Die Gateway Adresse
extern union LONG_BYTE DNSserver;                                           // Die DNS Server Adresse
extern unsigned long DNS_Answ_IP;                                           // Nach einem DNS Request steht hier die IP der angefragten adresse
// ***********************************************************************************************************************

// FUNCTIONS *************************************************************************************************************
unsigned char DNSRequest(char *HOSTNAME);                          //
void DNSAnswer(void);                                                       //
unsigned int DNS_convertHostName( char * HOSTNAME);                //
// ***********************************************************************************************************************

// DNS Struct ************************************************************************************************************
struct DNS_request                                                          //
{                                                                           //
    unsigned int TransactionID;                                             // 2 byte
    unsigned int Flags;                                                     // 2 byte
    unsigned int Questions;                                                 // 2 byte
    unsigned int Answer_RRs;                                                // 2 byte
    unsigned int Authority_RRs;                                             // 2 byte
    unsigned int Additional_RRs;                                            // 2 byte  = 12 Bytes
};                                                                          //
// ***********************************************************************************************************************

// DNS Answer Struct *****************************************************************************************************
struct DNS_answer                                                           //
{                                                                           //
    unsigned int Name;                                                      // 2 byte
    unsigned int Type;                                                      // 2 byte
    unsigned int IN;                                                        // 2 byte
    unsigned long TTL;                                                      // 4 byte
    unsigned int Datalenght;                                                // 2 byte = 12 Bytes
    unsigned long Adress;                                                   // 4 byte = 16 Bytes
};                                                                          //
// ***********************************************************************************************************************
#endif
