// ***********************************************************************************************************************
// ** File:         icmp.c                                                                                              **
// ** Title:        Routine for ICMP Protocol                                                                           **
// ** CPU:          ATMega644                                                                                           **
// ** Created:      28.02.2008                                                                                          **
// ** Revised:      28.02.2008                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "icmp.h"                                                           //
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// ICMP Pakete beantworten ***********************************************************************************************
void icmp( unsigned int packet_lenght )                                     //
{                                                                           //
    unsigned int checksum;                                                  //
    struct ETH_header *ETH_packet;                                          // ETH_struct anlegen
    struct IP_header *IP_packet;                                            // IP_struct anlegen
    struct ICMP_header *ICMP_packet;                                        // ICMP_struct anlegen
    ETH_packet = (struct ETH_header *)&buffer;                              //
    IP_packet = (struct IP_header *)&buffer[ETHERNET_HEADER_LENGTH];        //
    ICMP_packet = (struct ICMP_header *)&buffer[ETHERNET_HEADER_LENGTH +    //
        IP_HEADER_LENGTH];                                                  //
                                                                            //
    switch ( ICMP_packet->ICMP_type )                                       //
    {                                                                       //
        case 0x08:                                                          // IP_header unbauen zum versenden
            IP_packet->IP_DestinationIP = IP_packet->IP_SourceIP;           // IP einsetzen
            IP_packet->IP_SourceIP = MyIP.LONG;                             // IP einsetzen
            IP_packet->IP_Headerchecksum = 0x00;                            // IP_header checksumme ausrechnen
            checksum = Checksum_16( &buffer[14], IP_HEADER_LENGTH);         // IP_header checksumme ausrechnen
            IP_packet->IP_Headerchecksum = ChangeEndian16bit( checksum );   //
            ICMP_packet->ICMP_type = 0x00;                                  // auf reply umstellen
            ICMP_packet->ICMP_code = 0x00;                                  //
            if(ICMP_packet->ICMP_checksumByteOne >  0xFF-0x08)              // Simple ICMP Checksummenbildung, die Idee stammt von
            {                                                               // Simon, siehe http://avr.auctionant.de/
                ICMP_packet->ICMP_checksumByteTwo++;                        //
            }                                                               //
            ICMP_packet->ICMP_checksumByteOne+=0x08;                        //
            MakeETHheader(ETH_packet->ETH_sourceMac);                       // Ethernetframe bauen
            enc28j60PacketSend( packet_lenght, buffer);                     // packet_lenght - 4 weil der Controller die checksumme selber berechnet
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("ICMP->Send Reply to: ");                          //
            print_uart_ip (IP_packet->IP_DestinationIP);                    //
            #endif                                                          // DEBUG **************************************
        break;                                                              //
    }                                                                       //
}                                                                           //
// ***********************************************************************************************************************
