// ***********************************************************************************************************************
// ** File:         ip.c                                                                                                **
// ** Title:        Routine for IP Protocol                                                                             **
// ** CPU:          ATMega644                                                                                           **
// ** Created:      28.02.2008                                                                                          **
// ** Revised:      28.02.2008                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "ip.h"                                                             //
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// IP Header umbauen *****************************************************************************************************
void MakeIPheader(unsigned long SourceIP,unsigned char Protocoll,unsigned int Datalenght)//
{                                                                           //
    unsigned int checksum;                                                  //
    struct IP_header *IP_packet;                                            // IP_struct anlegen
    IP_packet = (struct IP_header *)&buffer[ETHERNET_HEADER_LENGTH];        //
                                                                            //
    IP_packet->IP_Version_Headerlen = 0x45;                                 //
    IP_packet->IP_TOS = 0x00;                                               //
    IP_packet->IP_Totallenght = ChangeEndian16bit(IP_HEADER_LENGTH + Datalenght);//
    IP_packet->IP_Identification = 0x1DAC;                                  //
    IP_packet->IP_Flags = 0x40;                                             //
    IP_packet->IP_Fragmentoffset = 0x00;                                    //
    IP_packet->IP_TTL = 0x80;                                               //
    IP_packet->IP_Protocol = Protocoll;                                     //
    IP_packet->IP_Headerchecksum = 0x00;                                    //
    IP_packet->IP_SourceIP = MyIP.LONG;                                     //
    IP_packet->IP_DestinationIP = SourceIP;                                 //
    checksum = Checksum_16( &buffer[14], IP_HEADER_LENGTH);                 // IP_header checksumme ausrechnen
    IP_packet->IP_Headerchecksum = ChangeEndian16bit( checksum );           //
}                                                                           //
// ***********************************************************************************************************************

// Pseudo Header bauen ***************************************************************************************************
void MakePseudoHeader (unsigned long SourceIP,unsigned char Protocoll,unsigned int Datalenght)
{                                                                           //
    struct IP_Pseudoheader *IP_pseudopacket;                                // IP_Pseudoheader struct anlegen
    IP_pseudopacket = (struct IP_Pseudoheader *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH -   //
        IP_PSEUDOHEADER_LENGHT];                                            //
                                                                            //
    IP_pseudopacket->IP_SourceIP = MyIP.LONG;                               //
    IP_pseudopacket->IP_DestinationIP = SourceIP;                           //
    IP_pseudopacket->IP_ZERO = 0x00;                                        //
    IP_pseudopacket->IP_Protokoll = Protocoll;                              //
    IP_pseudopacket->IP_lenght=ChangeEndian16bit(Datalenght);               //
}                                                                           //
// ***********************************************************************************************************************

// IP Paket auswerten ****************************************************************************************************
void ip( unsigned int packet_lenght )                                       //
{                                                                           //
    struct IP_header *IP_packet;                                            // IP_struct anlegen
    IP_packet = (struct IP_header *)&buffer[ETHERNET_HEADER_LENGTH];        //
    switch ( IP_packet->IP_Protocol )                                       //
    {                                                                       //
        case 0x01:                                                          // Wenn ICMP dann
            if ( IP_packet->IP_DestinationIP != MyIP.LONG )                 // prfe ob fr meine IP Adresse
            {                                                               //
                return;                                                     // wenn nicht dann abbrechen
            }                                                               //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug("IP->ICMP Packet received from: ");                 //
            print_uart_ip (IP_packet->IP_SourceIP);                         //
            #endif                                                          // DEBUG **************************************
            icmp( packet_lenght );                                          // wenn doch dann ICMP beantworten
        break;                                                              //
        case 0x06:                                                          //
            if ( IP_packet->IP_DestinationIP != MyIP.LONG )                 // prfe ob fr meine IP Adresse
            {                                                               //
                return;                                                     // wenn nicht dann abbrechen 
            }                                                               //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug("IP->TCP Packet received from: ");                  //
            print_uart_ip (IP_packet->IP_SourceIP);                         //
            #endif                                                          // DEBUG **************************************
            tcp();                                                          //
        break;                                                              //
        case 0x11:                                                          //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            if ( IP_packet->IP_DestinationIP == MyIP.LONG )                 // prfe ob fr meine IP Adresse
            {                                                               //
                print_debug("IP->UDP Packet received from: ");              //
                print_uart_ip (IP_packet->IP_SourceIP);                     //
            }                                                               //
            #endif                                                          // DEBUG **************************************
            udp();                                                          //
        break;                                                              //
    }                                                                       //
}                                                                           //
// ***********************************************************************************************************************
