// ***********************************************************************************************************************
// ** File:         sntp.c                                                                                              **
// ** Title:        Routine for Simple Network Time Protocol                                                            **
// ** CPU:          AT89C51ED2                                                                                          **
// ** Created:      23.04.2007                                                                                          **
// ** Revised:      23.04.2007                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "sntp.h"                                                           //
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
volatile unsigned char NTP_Flag;                                            //
// ***********************************************************************************************************************

// Time Paket auswerten **************************************************************************************************
void SNTP(void)                                                             //
{                                                                           //
    unsigned char i,tmp,tmp_year = 0,leap400 = 4,mtd_ptr = 0,mon_day = 0;   //
    unsigned int tmp_day;                                                   //
    union DATE ZeitInSek;                                                   //
    for ( i = 0 ; i < 4 ; i++ )                                             // Zeitinformation aus Buffer holen
    {                                                                       //
        ZeitInSek.DateByte[i] = buffer[ETHERNET_HEADER_LENGTH +             // und in ZeitInSek abspeichern
            IP_HEADER_LENGTH + UDP_HEADER_LENGHT + 3 - i];                  //
    }                                                                       //
    ZeitInSek.Date = ZeitInSek.Date + TimeZone;                             // Zeitinformation + Timezone
    if (SZ_Flag == SZ_SET)                                                  // Wenn Sommerzeitflag gesetzt dann
    {                                                                       //
        ZeitInSek.Date = ZeitInSek.Date + 3600;                             // Sommerzeit 1 Stunde hinzuaddieren 
    }                                                                       //
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_uart_clock();                                                     // Uhr ber Uart ausgeben
    printf_fast ("SNTP->Received New Timestamp: %lu\r\n",ZeitInSek.Date);   // Debug ausgabe
    #endif                                                                  // DEBUG **************************************
    seconds = ZeitInSek.Date % 60;                                          // Sekunden sind der rest wenn durch 60 geteilt wird
    ZeitInSek.Date = ZeitInSek.Date / 60;                                   // Minuten die seit dem 1.1.1900 vergangen sind
    minutes = ZeitInSek.Date % 60;                                          // Minuten sind der rest wenn durch 60 geteilt wird
    ZeitInSek.Date = ZeitInSek.Date / 60;                                   // Stunden die seit dem 1.1.1900 vergangen sind
    hours = ZeitInSek.Date % 24;                                            // Stunden sind der rest wenn durch 60 geteilt wird
    ZeitInSek.Date = ZeitInSek.Date / 24;                                   // Tage die seit dem 1.1.1900 vergangen sind
    wday = ZeitInSek.Date % DaysPerWeek;                                    // Tage die seit dem 1.1.1900 vergangen sind / 7 rest ist der Wochentag ( 0 = Montag )
    while(1)                                                                // FIXME: add 400yr exception handling !
    {                                                                       //
        tmp_day = 365;                                                      // Jahr hat 365 Tage
        if((tmp_year & 0x03)==0)                                            // Alle 4 Jahre hat das Jahr
        {                                                                   //
            tmp_day = 365 + 1;                                              // 366 Tage
                                                                            //
            if(tmp_year == 0 || tmp_year == 100 || tmp_year == 200)         // Wenn das Jahr 1900 oder 2000 oder 2100 ist dann gibt es kein Schaltjahr
            {                                                               //
                if (--leap400)                                              //
                {                                                           //
                    tmp_day = 365;                                          // und das Jahr hat 365 Tage
                }                                                           //
            }                                                               //
        }                                                                   //
        if( ZeitInSek.Date < tmp_day )                                      // Sind weniger Tage im Zeitstempel dann
        {                                                                   //
            break;                                                          // Schleife abbrechen
        }                                                                   //
        ZeitInSek.Date -= tmp_day;                                          // Tage des Jahres vom Zeitstempel abziehen
        tmp_year++;                                                         // Jahreszhler inkrementieren
    }                                                                       //
    year = tmp_year + 1900;                                                 // Wenn tmp_day < 365 oder 366 ist dann ist das aktuelle Jahr in year
                                                                            // Zeitstempel hat jetzt die "verbrauchten" Tage des aktuellen Jahres
    if ((tmp_day == 365) && ZeitInSek.Date > (30+28))                       // passed. -> calc month & day
    {                                                                       //
        ZeitInSek.Date++;                                                   // no leap year ! -> skip 29th feb
    }                                                                       //
    for(tmp=1; tmp<12; tmp++)                                               // Monat ausrechnen
    {                                                                       //
        mon_day = Month_to_days[mtd_ptr++];                                 // Tabelle  31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
        ZeitInSek.Date -= mon_day;                                          // Tage des Monats vom Zeitstempel abziehen
        if (ZeitInSek.Date < mon_day)                                       // wenn der Zeitstempel < als der selekt. Monat tage hat
        {                                                                   //
            break;                                                          // dann schleife abrechen
        }                                                                   //
    }                                                                       //
    month = tmp+1;                                                          // Der aktuelle Monat ist tmp + 1
    day = ZeitInSek.Date;                                                   // Der rest im zeitstempel ist der Tag des Monats
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_uart_clock();                                                     // Uhr ber Uart ausgeben
    printf_fast ("SNTP->New Time: %s %02d.%s %d   %02d:%02d:%02d\r\n",      //
        Day_to_wdays[wday],day,Month_to_string[month-1],year,               //
        hours,minutes,seconds);                                             // Debug ausgabe
    #endif                                                                  // DEBUG **************************************
    NTP_Flag = NTP_ANSWER;                                                  //
}                                                                           //
// ***********************************************************************************************************************

// Zeitsynchronisation ausfhren *****************************************************************************************
unsigned char timesync(void)                                                //
{                                                                           //
    unsigned char i,j;                                                      //
    //char tmpbuffer[48];                                                     //
    struct ARP_TABLE *ARP_table;                                            //
    ARP_table = (struct ARP_TABLE *)arptablebuffer;                         //
    //EEProm_Read_String (tmpbuffer, EE_TP_Adr);                              // Timeserver aus dem EEProm lesen
    //i = DNSRequest(tmpbuffer);                                              // DNS Anfrage
    i = DNSRequest("time.fu-berlin.de");                                              // DNS Anfrage
    if (i==DNS_ANSWER)                                                      // wenn ok dann MAC auflsen
    {                                                                       //
        j = GetIP2MAC(Gateway.LONG);                                        // Gateway IP in Mac auflsen
        if (j==ARP_ANSWER)                                                  // wenn ok dann Time request
        {                                                                   //
            deltasec = 0;                                                   // Timeout Timer starten
            NTP_Flag = NO_NTP_ANSWER;                                       //
            udp_send(ARP_table->ARP_MAC,DNS_Answ_IP,0x0700,0x2500,0);       // mit Mac und IP Time Protokoll request starten
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("TIMESYNC->TP Request send\r\n");                  //
            #endif                                                          // DEBUG **************************************
            while (1)                                                       // Endlose Schleife
            {                                                               //
                ethernet();                                                 // Prfe ob ein Paket angekommen ist
                if ((NTP_Flag == NTP_ANSWER) && ( deltasec < TimeOut))      // Wenn die angefragte IP = angefragte IP und Timeout nicht erreicht ist
                {                                                           //
                    #ifdef _DEBUG                                           // DEBUG **************************************
                    print_debug ("TIMESYNC->TP Request OK\r\n");            //
                    #endif                                                  // DEBUG **************************************
                    return( NTP_ANSWER );                                   // Antwort auf unser Arp Paket angekommen
                }                                                           //
                if (deltasec >= TimeOut)                                    // Wenn der Timeout timer abgelaufen ist
                {                                                           //
                    #ifdef _DEBUG                                           // DEBUG **************************************
                    print_debug("TIMESYNC->TP Request Timeout\r\n");        //
                    #endif                                                  // DEBUG **************************************
                    return( NO_NTP_ANSWER );                                // Keine antwort bekommen
                }                                                           //
            }                                                               //
        }                                                                   //
        else                                                                //
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("TIMESYNC->No Arp Answer\r\n");                    //
            #endif                                                          // DEBUG **************************************
            return( NO_NTP_ANSWER );                                        //
        }                                                                   //
    }                                                                       //
    else                                                                    //
    {                                                                       //
        #ifdef _DEBUG                                                       // DEBUG **************************************
        print_debug ("TIMESYNC->No DNS Answer\r\n");                        //
        #endif                                                              // DEBUG **************************************
        return( NO_NTP_ANSWER );                                            //
    }                                                                       //
}                                                                           //
// ***********************************************************************************************************************
