// ***********************************************************************************************************************
// ** File:         tcp.c                                                                                               **
// ** Title:        Routine for TCP Protocol                                                                            **
// ** CPU:          AT89C51ED2                                                                                          **
// ** Created:      23.04.2007                                                                                          **
// ** Revised:      23.04.2007                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "tcp.h"                                                            //
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// TCP Header bauen ******************************************************************************************************
void MakeTCPHeader (unsigned long SourcePort,unsigned long AcknowledgeNumber, unsigned long SequenceNumber,unsigned char Flags,
    unsigned int Datalenght,unsigned char * Options)                        //
{                                                                           //
    unsigned int checksum;                                                  //
    struct TCP_header * TCP_packet;                                         // TCP struct anlegen
    TCP_packet = (struct TCP_header *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH];        //
    TCP_packet->TCP_DestinationPort = SourcePort;                           //
    TCP_packet->TCP_SourcePort = 0x5000;                                    // Port 80
    TCP_packet->TCP_AcknowledgeNumber = ChangeEndian32bit(AcknowledgeNumber);//
    TCP_packet->TCP_SequenceNumber = ChangeEndian32bit(SequenceNumber);     //
    TCP_packet->TCP_DataOffset = ((TCP_HEADER_LENGHT<<2)& 0xf0);            // Headergrsse setzen
    TCP_packet->TCP_ControllFlags = Flags;                                  //
    TCP_packet->TCP_Window = ChangeEndian16bit( MAX_FRAMELEN );             // Windowsize
    TCP_packet->TCP_Checksum = 0x0000;                                      // Checksumme auf 0
    TCP_packet->TCP_UrgentPointer = 0x00;                                   //
    TCP_packet->TCP_Options[0] = Options[0];                                //
    TCP_packet->TCP_Options[1] = Options[1];                                //
    TCP_packet->TCP_Options[2] = Options[2];                                //
    TCP_packet->TCP_Options[3] = Options[3];                                //
    checksum = Checksum_16( &buffer[22],TCP_HEADER_LENGHT + 12 + Datalenght);// TCP Paket + IP Pseudoheader + 2 Fllbytes checksumme ausrechnen
    TCP_packet->TCP_Checksum = ChangeEndian16bit( checksum );               //
}                                                                           //
// ***********************************************************************************************************************

// TCP Get Acknowledge Senden ********************************************************************************************
void send_tcp_get_ack ()                                                    //
{                                                                           //
    unsigned long TempL;                                                    //
    unsigned int d;                                                         //
    struct ETH_header *ETH_packet;                                          // ETH_struct anlegen
    struct IP_header *IP_packet;                                            // IP_struct anlegen
    struct TCP_header * TCP_packet;                                         // TCP struct anlegen
    struct IP_Pseudoheader *IP_pseudopacket;                                // IP_Pseudoheader struct anlegen
    ETH_packet = (struct ETH_header *)buffer;                                                    // ETH_packet start @ buffer[0]
    IP_packet = (struct IP_header *)&buffer[ETHERNET_HEADER_LENGTH];                            // IP_packet start @ buffer[14]
    TCP_packet = (struct TCP_header *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH];        // TCP_packet start @ buffer[34]
    IP_pseudopacket = (struct IP_Pseudoheader *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH -   // IP_pseudo start @ buffer[22]
        IP_PSEUDOHEADER_LENGHT];                                            //
    d =(ChangeEndian16bit(IP_packet->IP_Totallenght) - (IP_HEADER_LENGTH    // Wenn IP_Totallen -(IP_Headerlen + TCP_Headerlen) = 0 ist
        + (( TCP_packet->TCP_DataOffset & 0xf0 ) >> 2 ) ) ) ;               // dann enthlt das empfangene Paket keine Daten
    MakeETHheader(ETH_packet->ETH_sourceMac);                               // Ethernet Header umbauen
    MakePseudoHeader (IP_packet->IP_SourceIP,0x06,TCP_HEADER_LENGHT);       // Pseudoheader fr die Checksumme bauen
    TCP_packet->TCP_Options[0] = 0x00;                                      // Options auf 0
    TCP_packet->TCP_Options[1] = 0x00;                                      //
    TCP_packet->TCP_Options[2] = 0x00;                                      //
    TCP_packet->TCP_Options[3] = 0x00;                                      //
    TempL = ChangeEndian32bit(TCP_packet->TCP_AcknowledgeNumber);           // Emfangene ACK Nummer sichern
    MakeTCPHeader (TCP_packet->TCP_SourcePort,ChangeEndian32bit(            // TCP Header generieren als ACK Nummer wird die empf.
        TCP_packet->TCP_SequenceNumber)+ d ,TempL,TCP_ACK_FLAG,             // sequenznummer + die Anzalhl der empfangenen Daten ohne
        0,TCP_packet->TCP_Options);                                         // TCP Header zurckgesendet
    MakeIPheader(IP_pseudopacket->IP_DestinationIP,0x06,TCP_HEADER_LENGHT); // IP Header generieren
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_debug("SEND_TCP_GET_ACK->TCP Get Ack Packet send to: ");          //
    print_uart_ip (IP_packet->IP_DestinationIP);                            //
    #endif                                                                  // DEBUG **************************************
    enc28j60PacketSend( ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH           // ACK zurcksenden
        + TCP_HEADER_LENGHT,buffer);                                        //
}                                                                           //
// ***********************************************************************************************************************

// TCP Ack senden ********************************************************************************************************
void send_tcp_ack ()                                                        //
{                                                                           //
    unsigned long TempL;                                                    //
    struct ETH_header *ETH_packet;                                          // ETH_struct anlegen
    struct IP_header *IP_packet;                                            // IP_struct anlegen
    struct TCP_header * TCP_packet;                                         // TCP struct anlegen
    struct IP_Pseudoheader *IP_pseudopacket;                                // IP_Pseudoheader struct anlegen
    ETH_packet = (struct ETH_header *)buffer;                                                    // ETH_packet start @ buffer[0]
    IP_packet = (struct IP_header *)&buffer[ETHERNET_HEADER_LENGTH];                            // IP_packet start @ buffer[14]
    TCP_packet = (struct TCP_header *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH];        // TCP_packet start @ buffer[34]
    IP_pseudopacket = (struct IP_Pseudoheader *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH -   // IP_pseudo start @ buffer[22]
        IP_PSEUDOHEADER_LENGHT];                                            //
    MakeETHheader(ETH_packet->ETH_sourceMac);                               // Ethernet Header umbauen
    MakePseudoHeader (IP_packet->IP_SourceIP,0x06,TCP_HEADER_LENGHT);       // Pseudoheader fr die Checksumme bauen
    TCP_packet->TCP_Options[0] = 0x00;                                      // Options auf 0
    TCP_packet->TCP_Options[1] = 0x00;                                      //
    TCP_packet->TCP_Options[2] = 0x00;                                      //
    TCP_packet->TCP_Options[3] = 0x00;                                      //
    TempL = ChangeEndian32bit(TCP_packet->TCP_AcknowledgeNumber);           // Emfangene ACK Nummer sichern
    MakeTCPHeader (TCP_packet->TCP_SourcePort,ChangeEndian32bit(            // TCP Header generieren als ACK Nummer wird die empf.
        TCP_packet->TCP_SequenceNumber)+ 1 ,TempL,TCP_ACK_FLAG,             // sequenznummer + die Anzalhl der empfangenen Daten ohne
        0,TCP_packet->TCP_Options);                                         // TCP Header zurckgesendet
    MakeIPheader(IP_pseudopacket->IP_DestinationIP,0x06,TCP_HEADER_LENGHT); // IP Header generieren
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_debug("SEND_TCP_ACK->TCP Ack Packet send to: ");                  //
    print_uart_ip (IP_packet->IP_DestinationIP);                            //
    #endif                                                                  // DEBUG **************************************
    enc28j60PacketSend( ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH           // ACK zurcksenden
        + TCP_HEADER_LENGHT,buffer);                                        //
}                                                                           //
// ***********************************************************************************************************************

// TCP SYN Paket beantworten *********************************************************************************************
void send_tcp_syn_ack ()                                                    //
{                                                                           //
    struct ETH_header *ETH_packet;                                          // ETH_struct anlegen
    struct IP_header *IP_packet;                                            // IP_struct anlegen
    struct TCP_header * TCP_packet;                                         // TCP struct anlegen
    struct IP_Pseudoheader *IP_pseudopacket;                                // IP_Pseudoheader struct anlegen
    ETH_packet = (struct ETH_header *)buffer;                                                    // ETH_packet start @ buffer[0]
    IP_packet = (struct IP_header *)&buffer[ETHERNET_HEADER_LENGTH];                            // IP_packet start @ buffer[14]
    TCP_packet = (struct TCP_header *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH];        // TCP_packet start @ buffer[34]
    IP_pseudopacket = (struct IP_Pseudoheader *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH -   // IP_pseudo start @ buffer[22]
        IP_PSEUDOHEADER_LENGHT];                                            //
    MakeETHheader(ETH_packet->ETH_sourceMac);                               // Ethernet Header umbauen
    MakePseudoHeader (IP_packet->IP_SourceIP,0x06, TCP_HEADER_LENGHT);      // Pseudoheader fr die Checksumme bauen
    MakeTCPHeader (TCP_packet->TCP_SourcePort,ChangeEndian32bit(            // TCP Header mit eigener ACK Number generieren
        TCP_packet->TCP_SequenceNumber)+1,0x12345678,                       //
        TCP_SYN_FLAG | TCP_ACK_FLAG, 0,TCP_packet->TCP_Options);            //
    MakeIPheader(IP_pseudopacket->IP_DestinationIP,0x06,TCP_HEADER_LENGHT); // IP Header generieren
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_debug("SEND_TCP_SYN_ACK->TCP Syn Ack Packet send to: ");          //
    print_uart_ip (IP_packet->IP_DestinationIP);                            //
    #endif                                                                  // DEBUG **************************************
    enc28j60PacketSend( ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH           // SYN + ACK zurcksenden
        + TCP_HEADER_LENGHT,buffer);                                        //
    return;                                                                 //
}                                                                           //
// ***********************************************************************************************************************

// TCP SYN Paket Senden **************************************************************************************************
void send_tcp_syn (unsigned char *DestMACAddress,unsigned long DestIP,unsigned int DestPort)
{                                                                           //
    struct TCP_header * TCP_packet;                                         // TCP struct anlegen
    TCP_packet = (struct TCP_header *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH];        // TCP_packet start @ buffer[34]
    MakeETHheader(DestMACAddress);                                          // Ethernet Header umbauen
    MakePseudoHeader (DestIP,0x06, TCP_HEADER_LENGHT);                      // Pseudoheader fr die Checksumme bauen
    TCP_packet->TCP_Options[0] = 0x02;                                      // Options auf 0
    TCP_packet->TCP_Options[1] = 0x04;                                      //
    TCP_packet->TCP_Options[2] = 0x03;                                      //
    TCP_packet->TCP_Options[3] = 0xe8;                                      //
    MakeTCPHeader (DestPort,0x00000000,ChangeEndian32bit(0x23456789),       // TCP Header mit eigener ACK Number generieren
        TCP_SYN_FLAG, 0,TCP_packet->TCP_Options);                           //
    MakeIPheader(DestIP,0x06,TCP_HEADER_LENGHT);                            // IP Header generieren
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_debug("SEND_TCP_SYN->TCP Syn Packet send to: ");                  //
    print_uart_ip (DestIP);                                                 //
    #endif                                                                  // DEBUG **************************************
    enc28j60PacketSend( ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH           // SYN + ACK zurcksenden
        + TCP_HEADER_LENGHT,buffer);                                        //
    return;                                                                 //
}                                                                           //
// ***********************************************************************************************************************

// TCP FIN ACK beantworten ***********************************************************************************************
void send_tcp_fin_ack ()                                                    //
{                                                                           //
    unsigned long TempL;                                                    //
    struct ETH_header *ETH_packet;                                          // ETH_struct anlegen
    struct IP_header *IP_packet;                                            // IP_struct anlegen
    struct TCP_header * TCP_packet;                                         // TCP struct anlegen
    struct IP_Pseudoheader *IP_pseudopacket;                                // IP_Pseudoheader struct anlegen
    ETH_packet = (struct ETH_header *)buffer;                                                    // ETH_packet start @ buffer[0]
    IP_packet = (struct IP_header *)&buffer[ETHERNET_HEADER_LENGTH];                            // IP_packet start @ buffer[14]
    TCP_packet = (struct TCP_header *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH];        // TCP_packet start @ buffer[34]
    IP_pseudopacket = (struct IP_Pseudoheader *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH -   // IP_pseudo start @ buffer[22]
        IP_PSEUDOHEADER_LENGHT];                                            //
    MakeETHheader(ETH_packet->ETH_sourceMac);                               // Ethernet Header umbauen
    MakePseudoHeader (IP_packet->IP_SourceIP,0x06, TCP_HEADER_LENGHT);      // Pseudoheader fr die Checksumme bauen
    TCP_packet->TCP_Options[0] = 0x00;                                      // Options auf 0
    TCP_packet->TCP_Options[1] = 0x00;                                      //
    TCP_packet->TCP_Options[2] = 0x00;                                      //
    TCP_packet->TCP_Options[3] = 0x00;                                      //
    TempL = ChangeEndian32bit(TCP_packet->TCP_AcknowledgeNumber);           // Emfangene ACK Nummer sichern
    MakeTCPHeader (TCP_packet->TCP_SourcePort,ChangeEndian32bit(            // TCP Header generieren
        TCP_packet->TCP_SequenceNumber)+1,TempL,TCP_ACK_FLAG,               //
        0,TCP_packet->TCP_Options);                                         //
    MakeIPheader(IP_pseudopacket->IP_DestinationIP,0x06,TCP_HEADER_LENGHT); // IP Header generieren
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_debug("SEND_TCP_FIN_ACK->TCP Fin Ack Packet send to: ");          //
    print_uart_ip (IP_packet->IP_DestinationIP);                            //
    #endif                                                                  // DEBUG **************************************
    enc28j60PacketSend( ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH           // ACK zurcksenden
        + TCP_HEADER_LENGHT,buffer);                                        //
    return;                                                                 //
}                                                                           //
// ***********************************************************************************************************************

// TCP Daten Senden ******************************************************************************************************
void send_tcp_data (unsigned int datlen, unsigned int nextseq)              //
{                                                                           //
    struct IP_header *IP_packet;                                            // IP_struct anlegen
    struct TCP_header * TCP_packet;                                         // TCP struct anlegen
    struct IP_Pseudoheader *IP_pseudopacket;                                // IP_Pseudoheader struct anlegen
    IP_packet = (struct IP_header *)&buffer[ETHERNET_HEADER_LENGTH];                            // IP_packet start @ buffer[14]
    TCP_packet = (struct TCP_header *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH];        // TCP_packet start @ buffer[34]
    IP_pseudopacket = (struct IP_Pseudoheader *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH -   // IP_pseudo start @ buffer[22]
        IP_PSEUDOHEADER_LENGHT];                                            //
    MakePseudoHeader (IP_packet->IP_DestinationIP,0x06, TCP_HEADER_LENGHT + datlen);// Pseudoheader fr die Checksumme bauen
    MakeTCPHeader (TCP_packet->TCP_DestinationPort,ChangeEndian32bit(       // TCP Header generieren es wird die selbe Ack nummer gesendet
        TCP_packet->TCP_AcknowledgeNumber), ChangeEndian32bit(              // die Seq nummer wird um die lnge der bis jetzt gesendeten daten
        TCP_packet->TCP_SequenceNumber)+nextseq,                            // erhht
        TCP_PSH_FLAG | TCP_ACK_FLAG, datlen,TCP_packet->TCP_Options);       //
    MakeIPheader(IP_pseudopacket->IP_DestinationIP,0x06,                    // IP Header generieren
        TCP_HEADER_LENGHT+datlen);                                          //
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_debug("SEND_TCP_DATA->TCP Data Packet send to: ");                //
    print_uart_ip (IP_packet->IP_DestinationIP);                            //
    #endif                                                                  // DEBUG **************************************
    enc28j60PacketSend( ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH           // ACK + PSH zurcksenden
        + TCP_HEADER_LENGHT+datlen,buffer);                                 //
}                                                                           //
// ***********************************************************************************************************************

// TCP verbindung beenden ************************************************************************************************
void send_tcp_fin (unsigned int datlen)                                     //
{                                                                           //
    struct IP_header *IP_packet;                                            // IP_struct anlegen
    struct TCP_header * TCP_packet;                                         // TCP struct anlegen
    struct IP_Pseudoheader *IP_pseudopacket;                                // IP_Pseudoheader struct anlegen
    IP_packet = (struct IP_header *)&buffer[ETHERNET_HEADER_LENGTH];                            // IP_packet start @ buffer[14]
    TCP_packet = (struct TCP_header *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH];        // TCP_packet start @ buffer[34]
    IP_pseudopacket = (struct IP_Pseudoheader *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH -   // IP_pseudo start @ buffer[22]
        IP_PSEUDOHEADER_LENGHT];                                            //
    MakePseudoHeader (IP_packet->IP_DestinationIP,0x06, TCP_HEADER_LENGHT); // Pseudoheader fr die Checksumme bauen
    MakeTCPHeader (TCP_packet->TCP_DestinationPort,ChangeEndian32bit(       // TCP Header generieren es wird die selbe Ack nummer gesendet
        TCP_packet->TCP_AcknowledgeNumber), ChangeEndian32bit(              // die Seq nummer wird um die lnge der bis jetzt gesendeten daten
        TCP_packet->TCP_SequenceNumber)+(datlen),                           // erhht
        TCP_ACK_FLAG | TCP_FIN_FLAG,                                        //
        0,TCP_packet->TCP_Options);                                         //
    MakeIPheader(IP_pseudopacket->IP_DestinationIP,0x06,                    // IP Header generieren
        TCP_HEADER_LENGHT);                                                 //
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_debug("SEND_TCP_FIN->TCP Fin Packet send to: ");                  //
    print_uart_ip (IP_packet->IP_DestinationIP);                            //
    #endif                                                                  // DEBUG **************************************
    enc28j60PacketSend( ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH           // ACK + FIN zurcksenden
        + TCP_HEADER_LENGHT,buffer);                                        //
}                                                                           //
// ***********************************************************************************************************************

// TCP Daten beantworten *************************************************************************************************
void tcp()                                                                  //
{                                                                           //
    unsigned int d, seq_no, temp_len;                                       //
    unsigned char *TCP_DATA_Buffer;                                         //
    struct IP_header *IP_packet;                                            // IP_struct anlegen
    struct TCP_header * TCP_packet;                                         // TCP struct anlegen
                                                                            //
    IP_packet = (struct IP_header *)&buffer[ETHERNET_HEADER_LENGTH];        // IP_packet start @ buffer[14]
    TCP_packet = (struct TCP_header *)&buffer[ETHERNET_HEADER_LENGTH +      //
        IP_HEADER_LENGTH];                                                  // TCP_packet start @ buffer[34]
    TCP_DATA_Buffer = &buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH +   //
        (( TCP_packet->TCP_DataOffset & 0xf0)>>2)];                         // TCP Data start @ buffer[58]
                                                                            //
    if ( TCP_packet->TCP_ControllFlags == TCP_SYN_FLAG )                    // Ist das ankommende Paket ein SYN Paket
    {                                                                       //
        send_tcp_syn_ack ();                                                // SYN mit SYN ACK beantworten
        return;                                                             //
    }                                                                       //
    if ( TCP_packet->TCP_ControllFlags == TCP_ACK_FLAG + TCP_SYN_FLAG )     // Ist das ankommende Paket ein SYN ACK Paket
    {                                                                       //
        
        send_tcp_ack ();                                                    // SYN ACK mit ACK beantworten
        return;                                                             //
    }                                                                       //
    if ( TCP_packet->TCP_ControllFlags == TCP_ACK_FLAG + TCP_FIN_FLAG )     // Ist das ankommende Paket ein Ack auf unser FIN Paket
    {                                                                       //
        send_tcp_fin_ack ();                                                // FIN ACK mit ACK beantworten
        return;                                                             //
    }                                                                       //
    d = (ChangeEndian16bit(IP_packet->IP_Totallenght) - (IP_HEADER_LENGTH   // Wenn IP_Totallen -(IP_Headerlen + TCP_Headerlen) = 0 ist
            + (( TCP_packet->TCP_DataOffset & 0xf0 )>>2)));                 // dann enthlt das empfangene Paket keine Daten
    if ( d != 0 )                                                           // Enthlt das angekommene Paket Daten (GET /) ?
    {                                                                       //
        
        if (( memcmp( TCP_DATA_Buffer , "GET / HTTP/" , 11 ) == 0 ) ||      // Wenn die ersten Zeichen im TCP_DataBuffer = GET / HTTP/ 
            ( memcmp( TCP_DATA_Buffer , "GET /main.html" , 14 ) == 0 ))     // oder GET /main.htm sind
        {                                                                   //
        
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("HTTP->GET /main.html\r\n");                       //
            #endif                                                          // DEBUG **************************************
            send_tcp_get_ack ();                                            // Sendet den ACK fr das Empfangene GET Kommando
            //seq_no = http_data_out (0, 0);                             // rckgabewert = grsse des letzten Paketes
            seq_no = http_data_out (HtmlTest,0);
            //temp_len = http_data_out (1, 0);                             // rckgabewert = grsse des letzten Paketes
            //seq_no = http_data_out (2, temp_len);                             // rckgabewert = grsse des letzten Paketes
            /*
            temp_len = http_data_out (HtmlKopf1, 0);                        // rckgabewert = grsse des letzten Paketes
            seq_no = http_data_out (HtmlMain1, temp_len);                   // rckgabewert = grsse des letzten Paketes
            */
            send_tcp_fin (seq_no);                                          // Sendet FIN mit der lnge des letzten Datenpaket
            return;                                                         //
        }                                                                   //
        /*
        if ( memcmp( TCP_DATA_Buffer , "GET /io.html" , 12 ) == 0 )         // Wenn die ersten Zeichen im TCP_DataBuffer = GET /io.htm sind
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("HTTP->GET /io.html\r\n");                         //
            #endif                                                          // DEBUG **************************************
            http_io ();                                                     //
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer , "GET /iodata.html" , 16 ) == 0 )     // Wenn die ersten Zeichen im TCP_DataBuffer = GET /io.htm sind
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("HTTP->GET /iodata.html\r\n");                     //
            #endif                                                          // DEBUG **************************************
            http_io_data (TCP_DATA_Buffer + 16);                            //
            http_dyndata ();                                                //
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer , "GET /setup.html" , 15 ) == 0 )      // Wenn die ersten Zeichen im TCP_DataBuffer = GET / HTTP/ sind
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("HTTP->GET /setup.html\r\n");                      //
            #endif                                                          // DEBUG **************************************
            http_setup ();                                                  //
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer , "GET /mail.html" , 14 ) == 0 )       // Wenn die ersten Zeichen im TCP_DataBuffer = GET /io.htm sind
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("HTTP->GET /mail.html\r\n");                       //
            #endif                                                          // DEBUG **************************************
            http_mail ();                                                   //
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer , "GET /dyndata.html" , 17 ) == 0 )    // Wenn die ersten Zeichen im TCP_DataBuffer = GET / HTTP/ sind
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("HTTP->GET /dyndata.html\r\n");                    //
            #endif                                                          // DEBUG **************************************
            http_dyndata ();                                                //
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer , "GET /configip.html" , 18 ) == 0 )   // Wenn die ersten Zeichen im TCP_DataBuffer = GET / HTTP/ sind
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("HTTP->GET /configip.html\r\n");                   //
            #endif                                                          // DEBUG **************************************
            http_configip ();                                               //
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer , "GET /ipdata.html" , 16 ) == 0 )     // Wenn die ersten Zeichen im TCP_DataBuffer = GET / HTTP/ sind
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("HTTP->GET /ipdata.html\r\n");                     //
            #endif                                                          // DEBUG **************************************
            http_setup_data(TCP_DATA_Buffer + 16);                          // Parameter auswerten
            http_configip ();                                               // Neue parameter zurcksenden
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer , "GET /maildata.html" , 18 ) == 0 )   // Wenn die ersten Zeichen im TCP_DataBuffer = GET / HTTP/ sind
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("HTTP->GET /maildata.html\r\n");                   //
            #endif                                                          // DEBUG **************************************
            http_mail_data(TCP_DATA_Buffer + 18);                           // Parameter auswerten
//            http_configmail ();                                             // Neue parameter zurcksenden
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer , "GET /configmail.html" , 20 ) == 0 ) // Wenn die ersten Zeichen im TCP_DataBuffer = GET / HTTP/ sind
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("HTTP->GET /configmail.html\r\n");                 //
            #endif                                                          // DEBUG **************************************
            //http_configmail ();                                             //
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer , "GET /formate.css" , 16 ) == 0 )     // Wenn die ersten Zeichen im TCP_DataBuffer = GET /io.htm sind
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("HTTP->GET /formate.css\r\n");                     //
            #endif                                                          // DEBUG **************************************
            send_tcp_get_ack ();                                            // Sendet den ACK fr das Empfangene GET Kommando
            seq_no = http_data_out (CssFormate1, 0);                        // rckgabewert = grsse des letzten Paketes
            send_tcp_fin (seq_no);                                          // Sendet FIN mit der lnge des letzten Datenpaket
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer , "GET /ajax.js" , 12 ) == 0 )         // Wenn die ersten Zeichen im TCP_DataBuffer = GET / HTTP/ sind
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("HTTP->GET /ajax.js\r\n");                         //
            #endif                                                          // DEBUG **************************************
            send_tcp_get_ack ();                                            // Sendet den ACK fr das Empfangene GET Kommando
            seq_no = http_data_out (JavaAjax, 0);                           // rckgabewert = grsse des letzten Paketes
            send_tcp_fin (seq_no);                                          // Sendet FIN mit der lnge des letzten Datenpaket
            return;                                                         //
        } 
                */
        if ( memcmp( TCP_DATA_Buffer , "GET /favicon.ico" , 16 ) == 0 )     // Wenn die ersten Zeichen im TCP_DataBuffer = GET / HTTP/ sind
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("HTTP->GET /favicon.ico\r\n");                     //    
            #endif                                                          // DEBUG **************************************
            send_tcp_get_ack ();                                            // Sendet den ACK fr das Empfangene GET Kommando
            seq_no = http_data_out (8, 0);                           // rckgabewert = grsse des letzten Paketes
            send_tcp_fin (seq_no);                                          // Sendet FIN mit der lnge des letzten Datenpaket
            return;                                                         //
        }                                                                   //                                                                  //

        
        /*
        if ( memcmp( TCP_DATA_Buffer , "GET /favicon.ico" , 16 ) == 0 )     // Wenn die ersten Zeichen im TCP_DataBuffer = GET / HTTP/ sind
        {                                                                   //
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("HTTP->GET /favicon.ico\r\n");                     //    
            #endif                                                          // DEBUG **************************************
            http_favicon ();                                                //
            return;                                                         //
        }                                                                   //
        */
        
        /*
// ********************************* MAILEMPFANG *********************************************************************************
        if ( memcmp( TCP_DATA_Buffer, "220 " , 4 ) == 0 )                   // Wenn die ersten Zeichen im TCP_DataBuffer = 220
        {                                                                   //
            //SMTP_Flag = SMTP_220;                                           // SMTP Server Identification
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer, "250 " , 4 ) == 0 )                   // Wenn die ersten Zeichen im TCP_DataBuffer = 250
        {                                                                   // 250 HELO OK
            //SMTP_Flag = SMTP_250;                                           // 250 Sender OK
            return;                                                         // 250 Recipient OK
        }                                                                   // 250 Message accepted
        if ( memcmp( TCP_DATA_Buffer, "354 " , 4 ) == 0 )                   // Wenn die ersten Zeichen im TCP_DataBuffer = 354
        {                                                                   // 354 Start of Mailtext
            //SMTP_Flag = SMTP_354;                                           //
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer, "334 " , 4 ) == 0 )                   // Wenn die ersten Zeichen im TCP_DataBuffer = 354
        {                                                                   // 354 Start of Mailtext
            //SMTP_Flag = SMTP_334;                                           //
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer, "235 " , 4 ) == 0 )                   // Wenn die ersten Zeichen im TCP_DataBuffer = 354
        {                                                                   // 354 Start of Mailtext
            //SMTP_Flag = SMTP_235;                                           //
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer, "221 " , 4 ) == 0 )                   // Wenn die ersten Zeichen im TCP_DataBuffer = 221
        {                                                                   //
            //SMTP_Flag = SMTP_221;                                           // 221 End of SMTP connection
            return;                                                         //
        }                                                                   //
        if ( memcmp( TCP_DATA_Buffer, "+OK " , 4 ) == 0 )                   // Wenn die ersten Zeichen im TCP_DataBuffer = 221
        {                                                                   //
            //POP_Flag = POP_OK;                                              // "+OK " ok vom Pop server
            return;                                                         //
        }                                                                   //
        */
    }                                                                       //
}                                                                           //
// ***********************************************************************************************************************
