// ***********************************************************************************************************************
// ** File:         tcp.h                                                                                               **
// ** Title:        Routine for TCP Protocol                                                                            **
// ** CPU:          AT89C51ED2                                                                                          **
// ** Created:      23.04.2007                                                                                          **
// ** Revised:      23.04.2007                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// COMPILER INFORMATION **************************************************************************************************
#ifndef TCP_H                                                               //
#define TCP_H                                                               //
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "crc.h"                                                            //
#include "enc28j60.h"                                                       //
#include "ip.h"                                                             //
#include "http.h"                                                           //
#include "httpdata.h"                                                       //
#include "global.h"                                                         //
//#include "smtp.h"                                                           //
//#include "pop.h"                                                            //
//#include "debug.h"                                                          //
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
#define TCP_HEADER_LENGHT           24                                      //
#define TCP_NON_FLAG                0x00                                    // kein Flag gesetzt
#define TCP_FIN_FLAG                0x01                                    // Verbindung beenden gesenden/empfangen
#define TCP_SYN_FLAG                0x02                                    // Verbindung aufbauen gesenden/empfangen
#define TCP_RST_FLAG                0x04                                    // Reset gesenden/empfangen
#define TCP_PSH_FLAG                0x08                                    // Push gesendet/empfangen
#define TCP_ACK_FLAG                0x10                                    // Acknowledge gesendet/empfangen
#define TCP_URG_FLAG                0x20                                    // Urgentflag fr dringlichkeit gesetzt
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// FUNCTIONS *************************************************************************************************************
void MakeTCPHeader (unsigned long SourcePort,unsigned long AcknowledgeNumber,
    unsigned long SequenceNumber,unsigned char Flags,unsigned int Datalenght,
    unsigned char * Options);                                               //
void send_tcp_get_ack (void);                                               //
void send_tcp_ack (void);                                                   //
void send_tcp_syn_ack (void);                                               //
void send_tcp_syn (unsigned char *DestMACAddress,unsigned long DestIP,unsigned int DestPort);
void send_tcp_fin_ack (void);                                               //
void send_tcp_data (unsigned int datlen, unsigned int nextseq);             //
void send_tcp_fin (unsigned int datlen);                                    //
void tcp(void);                                                             //
// ***********************************************************************************************************************

// TCP Struct ************************************************************************************************************
struct TCP_header                                                           //
{                                                                           //
    volatile unsigned int TCP_SourcePort;                                   // 2 byte
    volatile unsigned int TCP_DestinationPort;                              // 2 byte
    volatile unsigned long TCP_SequenceNumber;                              // 4 Byte
    volatile unsigned long TCP_AcknowledgeNumber;                           // 4 Byte
    volatile unsigned char TCP_DataOffset;                                  // 1 Byte
    volatile unsigned char TCP_ControllFlags;                               // 1 Byte
    volatile unsigned int TCP_Window;                                       // 2 byte
    volatile unsigned int TCP_Checksum;                                     // 2 byte
    volatile unsigned int TCP_UrgentPointer;                                // 2 byte
    volatile unsigned char TCP_Options[4];                                  // 4 Byte = 24 Byte
};                                                                          //
// ***********************************************************************************************************************
#endif
