// ***********************************************************************************************************************
// ** File:         timer.c                                                                                             **
// ** Title:        Timer Routine for Webserver                                                                         **
// ** CPU:          AT89C51ED2                                                                                          **
// ** Created:      23.04.2007                                                                                          **
// ** Revised:      23.04.2007                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "timer.h"                                                          //
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
unsigned char timermsec = 0;                                                // Milisekundenspeicher
unsigned char seconds = 0;                                                  // Sekundenspeicher
unsigned char minutes = 0;                                                  // Minutenspeicher
unsigned char hours = 0;                                                    // Stundenspeicher
unsigned char day = 1;                                                      // Tagesspeicher
unsigned char wday = 0;                                                     // Wochentagspeicher Montag = 0
unsigned char month = 1;                                                    // Monatsspeicher
unsigned int year = 2007;                                                   // Jahresspeicher
volatile unsigned char deltasec = 0;                                        // Delta Sekunden fr Timeouts 
unsigned char SZ_Flag = SZ_SET;                                             //
// ***********************************************************************************************************************

// CONSTANTS *************************************************************************************************************
PROGMEM unsigned char Month_to_days[12] =                                   //
{                                                                           //
    31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31                          //
};                                                                          //
char *Month_to_string[12] PROGMEM =                                         //
{                                                                           //
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",                               //
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"                                //
};                                                                          //
char *Day_to_wdays[7] PROGMEM =                                             //
{                                                                           //
    "Mo","Di","Mi","Do","Fr","Sa","So"                                      //
};                                                                          //
// ***********************************************************************************************************************

// Timer 0 Int Service Routine starten ***********************************************************************************
void timer_1_start(void)                                                    //
{                                                                           //
    timermsec = 0;                                                          // Millisecundenzhler auf 0
    cli();                                                                  // Interrupts aus
    TIMSK1 |= ( 1<<TOIE1 );                                                 // Timer1 einstellungen setzen
    TCCR1B |= ( 1<<CS12 ) | ( 0<<CS11 ) | ( 1<<CS10 );                      // 
    TCNT1 = 65535 - ( ( 8000000 / 1024 ) / 100 );                           // TCNT1 = 65535 - ( ( F_CPU / 1024 ) / SECDIV ) ;
    sei();                                                                  // Interrupts ein
}                                                                           //
// ***********************************************************************************************************************

// ***********************************************************************************************************************
ISR( TIMER1_OVF_vect )                                                      //
{                                                                           // 
    unsigned char tmp_sreg;                                                 // temporaerer Speicher fuer das Statusregister
    tmp_sreg = SREG;                                                        // Statusregister (also auch das I-Flag darin) sichern
    cli();                                                                  // Interrupts global deaktivieren
    TCNT1 = 65535 - ( ( 8000000 / 1024 ) / 100 ) ;                          // TCNT1 = 65535 - ( ( F_CPU / 1024 ) / SECDIV )
    SREG = tmp_sreg;                                                        // Status-Register wieder herstellen somit auch das I-Flag auf gesicherten Zustand setzen
    sei();                                                                  // Interrupts ein
    
    timermsec++;                                                            // msec speicher inc
    if (timermsec == 100)                                                   // wenn 100 x Int dann
    {                                                                       //
        timermsec = 0;                                                      // msec speicher = 0
        seconds++;                                                          // sekundenspeicher inc
        LED = !LED;
        deltasec++;                                                         // deltasec inc fr timeouts
        if (deltasec == 200)                                                // wenn deltasec > 200 dann
        {                                                                   //
            deltasec = 0;                                                   // deltasec = 0
        }                                                                   //
        if (seconds == 60)                                                  // wenn 60 sec dann
        {                                                                   //
            seconds = 0;                                                    // sec speicher = 0
            minutes++;                                                      // minuten inc
            if (minutes == 60)                                              // wenn 60 minuten
            {                                                               //
                minutes = 0;                                                // minuten = 0
                hours++;                                                    // stunden inc
                if (hours == 24)                                            // wenn stunden = 24
                {                                                           //
                    hours = 0;                                              // stunden = 0
                    day++;                                                  // Tag erhhen
                    wday++;                                                 // Wochentag erhhen
                    if (day > Month_to_days[month-1])                       // Wenn Tag > als der akt. Monat tage hat
                    {                                                       //
                        day = 1;                                            // Tag auf 1
                        month++;                                            // Monat erhhen
                        if (month > 12)                                     // Wenn Monat > 12 dann
                        {                                                   //
                            month = 1;                                      // Monat = 1
                            year++;                                         // Jahr erhhen
                        }                                                   //
                    }                                                       //
                    if (wday > 6)                                           // Wenn Wochentag > 6 = Sonntag
                    {                                                       //
                        wday = 0;                                           // dann Wochentag = 0 = Montag
                    }                                                       //
                }                                                           //
            }                                                               //
        }                                                                   //
        if (month == 3 && wday == 6 && (day + 6) > 31 && hours == 2 &&      // Wenn Monat = Mrz und Wochentag = Sonntag und Tag des Monats +
            SZ_Flag == SZ_NOT_SET)                                          // 6 > 31 Tage und Uhrzeit = 2 Uhr und Sommerzeitflag nicht gesetzt dann
        {                                                                   //
            SZ_Flag = SZ_SET;                                               // Sommerzeitflag setzen 
            hours++;                                                        // 1 Stunde vor Stellen
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("TIMER_0_ISR->Set Summertime Flag\r\n");           //
            #endif                                                          // DEBUG **************************************
        }                                                                   // 
        if (month == 10 && wday == 6 && (day + 6) > 31 && hours == 3 &&     // Wenn Monat = Oktober und Wochentag = Sonntag und Tag des Monats +
            SZ_Flag == SZ_SET)                                              // 6 > 31 Tage und Uhrzeit = 2 Uhr und Sommerzeitflag gesetzt dann
        {                                                                   //
            SZ_Flag = SZ_NOT_SET;                                           // Sommerzeitflag lschen
            hours--;                                                        // 1 Stunde zurck Stellen
            #ifdef _DEBUG                                                   // DEBUG **************************************
            print_debug ("TIMER_0_ISR->Clear Summertime Flag\r\n");         //
            #endif                                                          // DEBUG **************************************
        }                                                                   //
    }                                                                       //
}
// ***********************************************************************************************************************
