// ***********************************************************************************************************************
// ** File:         udp.c                                                                                               **
// ** Title:        Routine for UDP Protocol                                                                            **
// ** CPU:          AT89C51ED2                                                                                          **
// ** Created:      23.04.2007                                                                                          **
// ** Revised:      23.04.2007                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "udp.h"                                                            //
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
unsigned char DestMACAddress[6];                                            // Buffer fr die Ziel MAC Adresse
// ***********************************************************************************************************************

// UDP Senden ************************************************************************************************************
void udp_send( unsigned char *DestMACAddress, unsigned long DestIPAddress,  //
    unsigned long SourcePort, unsigned long DestPort,unsigned int datalen)  //
{                                                                           //
    unsigned int checksum;                                                  //
    struct UDP_header * UDP_packet;                                         // UDP_struct anlegen
    UDP_packet = (struct UDP_header *)&buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH];        //
    MakeETHheader(DestMACAddress);                                          // Ethernet Header umbauen 
    MakePseudoHeader(DestIPAddress, 0x11, UDP_HEADER_LENGHT + datalen );    // Pseudoheader fr UDP CRC 
    UDP_packet->UDP_Checksum = 0x0000;                                      // UDP CRC auf 0
    UDP_packet->UDP_DestinationPort = DestPort;                             // Zielport eintragen
    UDP_packet->UDP_SourcePort = SourcePort;                                // Quellport
    UDP_packet->UDP_Datalenght=ChangeEndian16bit(UDP_HEADER_LENGHT+datalen);// UDP Datalength = UDP_Header + Stringlnge
    checksum = Checksum_16(&buffer[22], UDP_HEADER_LENGHT + datalen + 12);  // UDP Paket + IP Pseudoheader checksumme ausrechnen
    UDP_packet->UDP_Checksum = ChangeEndian16bit( checksum );               // UDP Checksumme eintragen
    MakeIPheader( DestIPAddress, 0x11, UDP_HEADER_LENGHT + datalen );       // IP Header bauen
    #ifdef _DEBUG                                                           // DEBUG **************************************
    print_debug("UDP_SEND->Send UDP Packet to: ");                          //
    print_uart_ip (DestIPAddress);                                          //
    #endif                                                                  // DEBUG **************************************
    enc28j60PacketSend(ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH + UDP_HEADER_LENGHT// UDP Paket zurcksenden
        + datalen,buffer);                                                  //
    return;                                                                 //
}                                                                           //
// ***********************************************************************************************************************

// UDP Beantworten *******************************************************************************************************
void udp()                                                                  //
{                                                                           //
    unsigned char *UDP_DATA_Buffer;                                         //
    struct ETH_header *ETH_packet;                                          // ETH_struct anlegen
    struct IP_header *IP_packet;                                            // IP_struct anlegen
    struct UDP_header * UDP_packet;                                         // UDP_struct anlegen
    struct IP_Pseudoheader *IP_pseudopacket;                                // IP_Pseudoheader struct anlegen
    ETH_packet = (struct ETH_header *)buffer;                               //
    IP_packet = (struct IP_header *)&buffer[ETHERNET_HEADER_LENGTH];        //
    UDP_packet = (struct UDP_header *)&buffer[ETHERNET_HEADER_LENGTH +      //
        IP_HEADER_LENGTH];                                                  //
    IP_pseudopacket = (struct IP_Pseudoheader *)&buffer[ETHERNET_HEADER_LENGTH +//
        IP_HEADER_LENGTH - IP_PSEUDOHEADER_LENGHT];                         //
    UDP_DATA_Buffer = &buffer[ETHERNET_HEADER_LENGTH + IP_HEADER_LENGTH +   //
        UDP_HEADER_LENGHT];                                                 //
                                                                            //
    if (UDP_packet->UDP_SourcePort == 0x3500)                               // Port 53 DNS Prot
    {                                                                       // Kommt ein UDP paket auf Port 53
        #ifdef _DEBUG                                                       // DEBUG **************************************
        print_debug("UDP->DNS Answer Packet received from: ");              //
        print_uart_ip (IP_packet->IP_SourceIP);                             //
        #endif                                                              // DEBUG **************************************
        //DNSAnswer();                                                        // rein dann ist das eine DNS
        return;                                                             // antwort
    }                                                                       //
    if (UDP_packet->UDP_SourcePort == 0x2500)                               // Port 37 Time Prot
    {                                                                       // kommt ein UDP paket auf Port 37
        #ifdef _DEBUG                                                       // DEBUG **************************************
        print_debug("UDP->SNTP Answer Packet received from: ");             //
        print_uart_ip (IP_packet->IP_SourceIP);                             //
        #endif                                                              // DEBUG **************************************
        //SNTP();                                                             // rein dann ist das ein Time-
        return;                                                             // protokoll paket
    }                                                                       //
    if (UDP_packet->UDP_SourcePort == 0x4300)                               // Port 67 BootPS
    {                                                                       // kommt ein UDP paket auf Port 67
        #ifdef _DEBUG                                                       // DEBUG **************************************
        print_debug("UDP->DHCP Answer Packet received from: ");             //
        print_uart_ip (IP_packet->IP_SourceIP);                             //
        #endif                                                              // DEBUG **************************************
        DHCPAnswer();                                                       // rein dann ist das ein DHCP-
        return;                                                             // protokoll paket
    }                                                                       //
    return;                                                                 //
}                                                                           //
// ***********************************************************************************************************************
